﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Xml;
using System.Xml.Linq;
using System.Net;
using DisqusSharp;
using Thread = System.Threading.Thread;

namespace Prabir.DisqusImport
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void btnBrowse_Click(object sender, EventArgs e)
        {
            string xmlFile = BrowseFile();
            if (!string.IsNullOrEmpty(xmlFile))
                LoadFile(xmlFile);
        }

        private void LoadFile(string xmlFile)
        {
            if (!File.Exists(xmlFile))
            {
                MessageBox.Show(xmlFile + " doesn't exist.");
                return;
            }

            using (Stream stream = File.OpenRead(xmlFile))
            {
                using (StreamReader reader = new StreamReader(stream))
                {
                    txtContents.Text = reader.ReadToEnd();
                    FileName = xmlFile;
                }
            }
        }

        protected string BrowseFile()
        {
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.Multiselect = false;
            ofd.DefaultExt = "*.xml";

            if (ofd.ShowDialog() == DialogResult.OK)
                return ofd.FileName;
            return string.Empty;
        }

        public string FileName { get; set; }

        private void btnUploadToDisqus_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(FileName))
            {
                MessageBox.Show("Please Load the file first");
                return;
            }

            txtUserApiKey.ReadOnly = false;

            Forum forum = _forums[ddlWebsite.SelectedIndex];
            string forumApiKey = _disqus.GetForumApiKey(forum.ID);

            XDocument xdoc = XDocument.Load(FileName);

            Thread t = new Thread(() =>
                                      {
                                          int i = 0;
                                          foreach (XElement xElement in xdoc.Elements("articles").Nodes())
                                          {
                                              AppendStatusLine("Processing Article#: " + ++i);

                                              string url = xElement.Element("url").Value;

                                              DisqusThreadWithWebsite thread = _disqus.GetThreadByUrl(url, forumApiKey, string.Empty);

                                              if (thread == null)
                                                  continue;

                                              int threadId = thread.ID;

                                              AppendStatusLine("Thread Id: " + threadId);

                                              AppendStatusLine(url);
                                              AppendStatusLine();

                                              int j = 0;
                                              foreach (var comment in xElement.Elements("comments").Elements("comment"))
                                              {
                                                  AppendStatusLine("Uploading Comment# " + ++j);
                                                  string authorName = comment.Element("name").Value;
                                                  string email = comment.Element("email").Value;
                                                  string authorUrl = comment.Element("url").Value;
                                                  IPAddress ipAddress =
                                                      IPAddress.Parse(comment.Element("ip_address").Value);
                                                  string message = comment.Element("message").Value;
                                                  DateTime dateTime =
                                                      DateTime.ParseExact(comment.Element("date").Value,
                                                                          "ddd, dd MMM yyyy HH:mm:ss -0000", null);
                                                  int point = Convert.ToInt32(comment.Element("points").Value);

                                                  _disqus.CreatePost(forumApiKey, threadId,
                                                                     message, authorName, email, dateTime, ipAddress,
                                                                     authorUrl, null, null);

                                                  AppendStatusLine();
                                              }
                                          }
                                          txtUserApiKey.ReadOnly = true;

                                          MessageBox.Show("Your comments have been uploaded to Disqus.");
                                      });
            t.Start();



        }

        public void AppendStatus(string text)
        {
            txtStatus.Invoke(new MethodInvoker(() =>
            {
                txtStatus.Text += text;
            }));
        }

        public void AppendStatusLine(string text)
        {
            txtStatus.Invoke(new MethodInvoker(() =>
            {
                txtStatus.Text += text;
                txtStatus.Text += Environment.NewLine;
            }));
        }

        public void AppendStatusLine()
        {
            txtStatus.Invoke(new MethodInvoker(() =>
            {
                txtStatus.Text += Environment.NewLine;
            }));
        }

        private Disqus _disqus;
        private List<Forum> _forums;

        private void btnGetWebsites_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(txtUserApiKey.Text))
            {
                MessageBox.Show("Please enter uer api key. When you are logged in you can get your api key from http://disqus.com/api/get_my_key/");
                return;
            }

            _disqus = new Disqus(txtUserApiKey.Text);
            try
            {
                _forums = _disqus.GetForums();

                ddlWebsite.Items.Clear();

                foreach (Forum forum in _forums)
                    ddlWebsite.Items.Add(forum);

                if (_forums.Count > 0)
                    ddlWebsite.SelectedIndex = 0;
                else
                    MessageBox.Show("No websites found");

            }
            catch (Exception)
            {
                MessageBox.Show("Error occured. Are you sure you entered a valid api key?");
            }
        }
    }


}
