using System;
using System.Collections.Generic;
using System.Globalization;
using Newtonsoft.Json.Linq;

namespace DisqusSharp
{
    public class JsonConverters
    {
        #region forum

        public static List<Forum> ConvertGetForumList(string json)
        {
            JArray jArrayWebsites = JArray.Parse(json);

            List<Forum> forums = new List<Forum>();

            foreach (JToken jTokenWebsite in jArrayWebsites)
                forums.Add(GetForum(jTokenWebsite));

            return forums;
        }

        private static Forum GetForum(JToken jTokenForum)
        {
            JToken jcreatedAt = jTokenForum["created_at"];
            JToken jshortName = jTokenForum["shortname"];
            JToken jdescription = jTokenForum["description"];
            JToken jId = jTokenForum["id"];
            JToken jName = jTokenForum["name"];

            Forum forum = new Forum();

            if (jcreatedAt != null)
                forum.CreatedAt = ToDateTime(jcreatedAt.Value<string>());
            if (jshortName != null)
                forum.ShortName = jshortName.Value<string>();
            if (jdescription != null)
                forum.Description = jdescription.Value<string>();
            if (jId != null)
                forum.ID = jId.Value<int>();
            if (jName != null)
                forum.Name = jName.Value<string>();

            return forum;
        }

        #endregion

        #region category

        public static List<Category> ConvertGetCategoryList(string json)
        {
            JArray jCategories = JArray.Parse(json);

            List<Category> categories = new List<Category>();

            foreach (JToken jTokenWebsite in jCategories)
                categories.Add(GetCategory(jTokenWebsite));

            return categories;
        }

        private static Category GetCategory(JToken jTokencCategory)
        {
            JToken jisDefault = jTokencCategory["is_default"];
            JToken jTitle = jTokencCategory["title"];
            JToken jId = jTokencCategory["id"];
            JToken jForumId = jTokencCategory["forum_id"];

            Category forum = new Category();

            if (jisDefault != null)
                forum.IsDefault = jisDefault.Value<bool>();
            if (jTitle != null)
                forum.Title = jTitle.Value<string>();
            if (jId != null)
                forum.ID = jId.Value<int>();
            if (jForumId != null)
                forum.ForumID = jForumId.Value<int>();

            return forum;
        }

        #endregion

        #region thread

        public static List<Thread> ConvertGetThreadList(string json)
        {
            JArray jThreads = JArray.Parse(json);

            List<Thread> threads = new List<Thread>();

            foreach (JToken thread in jThreads)
                threads.Add(GetThread(thread));

            return threads;
        }

        private static Thread GetThread(JToken jTokenThread)
        {
            JToken jCategory = jTokenThread["category"];
            JToken jAllowComments = jTokenThread["allow_comments"];
            JToken jHidden = jTokenThread["hidden"];
            JToken jForumId = jTokenThread["forum_id"];
            JToken jTitle = jTokenThread["title"];
            JToken jUrl = jTokenThread["url"];
            JToken jCreatedAt = jTokenThread["created_at"];
            JToken jId = jTokenThread["id"];
            JToken jSlug = jTokenThread["slug"];

            Thread thread = new Thread();

            if (jCategory != null) thread.Category = GetCategory(jCategory);
            if (jAllowComments != null) thread.AllowComments = jAllowComments.Value<bool>();
            if (jHidden != null) thread.Hidden = jHidden.Value<bool>();
            if (jForumId != null) thread.ForumID = jForumId.Value<int>();
            if (jTitle != null) thread.Title = jTitle.Value<string>();
            if (jUrl != null) thread.Url = jUrl.Value<string>();
            if (jCreatedAt != null) thread.CreatedAt = ToDateTime(jCreatedAt.Value<string>());
            if (jId != null) thread.ID = jId.Value<int>();
            if (jSlug != null) thread.Slug = jSlug.Value<string>();

            return thread;
        }

        #endregion

        public static DateTime ToDateTime(string strDateTime)
        {
            // disqus seems to have different ways on presenting date/time.
            DateTime result;
            if (!DateTime.TryParseExact(strDateTime, "yyyy-MM-dd HH:mm:ss.ffffff", null, DateTimeStyles.None, out result))
            {
                if (!DateTime.TryParseExact(strDateTime, "yyyy-MM-ddTHH:mm", null, DateTimeStyles.None, out result))
                    throw new FormatException("String was not recognized as a valid date format.");
            }
            return result;
        }
    }
}